/*
 * Node-tar
 * MIT License (c) 2011 Tim Caswell, T. Jameson Little
 */

(function () {
	"use strict";

	require('futures/forEachAsync');
	
	var fs = require('fs'),
		Stream = require('stream').Stream,
		header = require("./header"),
		utils = require("./utils");
	
	
	function Tar() {
		var tape;

		Stream.apply(this, arguments);
		this.written = 0;

		tape = this;

		this.addListener('data', function (chunk) {
			tape.written += chunk.length;
		});
		this.addListener('end', function () {
			this.emit('data', utils.clean(10240 - (this.written % 10240)));
		});
	}
	
	Tar.prototype = Object.create(Stream.prototype, {
		constructor: { value: Tar }
	});
	
	Tar.prototype.close = function () {
		this.emit('end');
	};

	Tar.prototype.append = function (path, o, callback) {
		var tape = this;
	
		fs.lstat(path, function (err, stat) {
			var type, data = {}, mtime, filesize;
			function finish(cb) {
				var checksum = 0, written, input;
				Object.keys(data).forEach(function(key){
					var i, l,
						value = data[key];

					for (i = 0; i < value.length; i++) {
						checksum += value.charCodeAt(i);
					}
				});

				data.checksum = utils.pad(checksum, 6) + "\u0000 ";
				tape.emit('data', header.format(data));
				if (stat.isFile()) {
					written = 0;
					input = fs.createReadStream(path);
					input.on('data', function (chunk) {
						written += chunk.length;
						tape.emit('data', chunk);
					});
					input.on('error', function (err) {
						tape.emit('error', err);
						cb(err);
					});
					input.on('end', function () {
						tape.emit('data', utils.clean(512 - (written % 512)));
						cb();
					});
				} else {
					cb();
				}
			}

			if (err) {
				callback(err);
				return tape.emit('error', err);
			}

			try {
				if(stat.isDirectory()){
					path += "/";
					type = "5";
				}else if(stat.isSymbolicLink()){
					type = "2";
				}else if(stat.isFile()){
					type = "0";
				}else{
					type = new Error();
				}

				mtime = o.mtime || (stat.mtime.getTime() / 1000);
				filesize = o.fileSize || (stat.isFile() ? stat.size : 0);

				data = {
					fileName: o.fileName || path,
					fileMode: utils.pad((o.fileMode || (stat.mode & 0xfff)), 7),
					uid: utils.pad(o.uid || (stat.uid), 7),
					gid: utils.pad(o.gid || (stat.gid), 7),
					fileSize: utils.pad(filesize, 11),
					mtime: utils.pad(mtime, 11),
					checksum: "        ",
					type: type,
					ustar: "ustar  ",
					owner: o.owner || "",
					group: o.group || ""
				};

				if (stat.isSymbolicLink()) {
					fs.readlink(path, function (err, resolvedPath) {
						if (err) {
							callback(err);
							return tape.emit('error', err);
						}
						data.linkName = resolvedPath;	
						finish(callback);
					});
				} else {
					finish(callback);
				}
			} catch (err2) {
				tape.emit('error', err2);
				callback(err2);
			}
		});
	};

	Tar.prototype.addFiles = function (files, overrides, callback) {
		var error, tape = this;
		files.forEachAsync(function (next, filename) {
			// make sure we get through all cases
			if(error){
				next();
			}

			fs.lstat(filename, function (err, stat) {
				if (err) {
					error = err;
					next();
				}

				tape.append(filename, overrides, function(tErr){
					if(tErr){
						error = tErr;
						return next();
					}

					if (stat.isDirectory()) {
						tape.addDirectory(filename, overrides, function (tErr2) {
							if (tErr2) {
								error = tErr2;
								return next();
							}

							next();
						});
					}else{
						next();
					}
				});
			});
		}).then(function(){
			callback(error);
		});
	};

	Tar.prototype.addDirectory = function (path, overrides, callback) {
		var tape = this;
		fs.readdir(path, function (err, filenames) {
			if (err) {
				return callback(err);
			}

			filenames.forEach(function(filename, i, arr){
				arr[i] = path + "/" + filename;
			});

			return tape.addFiles.apply(tape, [filenames, overrides, callback]);
		});
	};
	
	module.exports = Tar;
}());
